#!/usr/bin/env node

// node_modules/.pnpm/tsup@5.12.0_typescript@4.6.2/node_modules/tsup/assets/esm_shims.js
import { fileURLToPath } from "url";
import path from "path";
var getFilename = () => fileURLToPath(import.meta.url);
var getDirname = () => path.dirname(getFilename());
var __dirname = /* @__PURE__ */ getDirname();

// src/index.ts
import path4 from "path";
import chalk2 from "chalk";
import { program } from "commander";
import { config } from "dotenv";

// src/dump.ts
import fs2 from "fs";
import path3 from "path";
import { Credential, LeetCode } from "leetcode-query";
import fetch from "node-fetch";
import Ora from "ora";

// src/constants.ts
var EXTS = {
  cpp: ".cpp",
  java: ".java",
  python: ".py",
  python3: ".py",
  c: ".c",
  csharp: ".cs",
  javascript: ".js",
  ruby: ".rb",
  swift: ".swift",
  golang: ".go",
  scala: ".scala",
  kotlin: ".kt",
  rust: ".rs",
  php: ".php",
  typescript: ".ts",
  racket: ".rkt",
  erlang: ".erl",
  elixir: ".ex"
};
var COMMENTS = {
  cpp: "//",
  java: "//",
  python: "#",
  python3: "#",
  c: "//",
  csharp: "//",
  javascript: "//",
  ruby: "#",
  swift: "//",
  golang: "//",
  scala: "//",
  kotlin: "//",
  rust: "//",
  php: "//",
  typescript: "//",
  racket: ";",
  erlang: "%",
  elixir: "#"
};
var LEETCODE_BASE = "https://leetcode.com";
var LEETCODE_DETAIL_BASE = LEETCODE_BASE + "/submissions/detail/";
var LEETCODE_SLUG_BASE = LEETCODE_BASE + "/problems/";
var README_TEMPLATE = `
# LeetCode Solutions

![LeetCode Stats Card]($card_url)

| Problem | Difficulty | Tags | Solution |
| ------- | ---------- | ---- | -------- |
$table
`;

// src/utils.ts
import fs from "fs";
import path2 from "path";
import chalk from "chalk";
function get_package_meta() {
  return JSON.parse(fs.readFileSync(path2.join(__dirname, "../package.json"), "utf8"));
}
function readable_memory(mem) {
  const units = ["B", "KB", "MB", "GB", "TB"];
  let unit = 0;
  while (mem > 1e3) {
    mem /= 1e3;
    unit++;
  }
  return `${mem.toFixed(2)} ${units[unit]}`;
}
function sleep(ms) {
  return new Promise((resolve) => setTimeout(resolve, ms));
}
async function retry(func, max_retry2 = 3) {
  let error = null;
  for (let retry2 = 0; retry2 < max_retry2; retry2++) {
    try {
      return await func();
    } catch (err) {
      console.error(chalk.redBright(err));
      error = err;
    }
  }
  throw error;
}

// src/dump.ts
var leetcode;
var max_retry = 3;
async function dump(session, output, clean, cooldown, timezone, pure, max) {
  process.env.TZ = timezone;
  max_retry = max;
  const dir = path3.resolve(output);
  if (fs2.existsSync(dir) && clean) {
    fs2.rmSync(dir, { recursive: true });
  }
  if (!fs2.existsSync(dir)) {
    fs2.mkdirSync(dir, { recursive: true });
  }
  const credential = new Credential();
  await credential.init(session);
  leetcode = new LeetCode(credential);
  const spinner = Ora({ text: "Scanning...", spinner: "bouncingBar" }).start();
  const { list, ac } = await get_list();
  spinner.succeed(`Scan Done. (${list.length} Problems, ${ac.length} Accepted)`);
  await sleep(cooldown);
  const table = [];
  spinner.start("Dumping Submissions...");
  for (let i = 0; i < ac.length; i++) {
    spinner.text = `Dumping Submissions... (${i + 1}/${ac.length})`;
    const { titleSlug } = ac[i];
    const problem = await get_problem(titleSlug);
    const folder = path3.resolve(dir, `${problem.questionFrontendId}. ${problem.title}`);
    if (!fs2.existsSync(folder)) {
      fs2.mkdirSync(folder, { recursive: true });
    }
    fs2.writeFileSync(path3.resolve(folder, "README.md"), `# ${problem.questionFrontendId}. ${problem.title}

Tags: ${problem.topicTags.map((t) => "`" + t.name + "`").join(", ")}

${problem.content}`);
    const submissions = await get_submissions(titleSlug);
    spinner.text += ` [${Object.keys(submissions).join(", ")}]`;
    await sleep(cooldown);
    const row = [
      `[${problem.questionFrontendId}. ${problem.title}](./${encodeURI(path3.basename(folder))}) [\u{1F517}](${LEETCODE_SLUG_BASE}${titleSlug}/)`,
      problem.difficulty,
      problem.topicTags.map((t) => "`" + t.name + "`").join(", "),
      /* @__PURE__ */ new Map()
    ];
    for (const lang in submissions) {
      const ext = EXTS[lang];
      const file = path3.resolve(folder, `${titleSlug}${ext}`);
      if (!fs2.existsSync(file)) {
        const submission = await get_submission(submissions[lang].id);
        const info = `${COMMENTS[lang]} ${problem.questionFrontendId}. ${problem.title} (${new Date(+submissions[lang].timestamp * 1e3).toLocaleDateString()})
${COMMENTS[lang]} Runtime: ${submission.runtime} ms (${submission.runtime_percentile.toFixed(2)}%) Memory: ${readable_memory(submission.memory)} (${submission.memory_percentile.toFixed(2)}%) 

`;
        fs2.writeFileSync(file, `${pure ? "" : info}${submission.code}`);
        row[3].set(lang, `./${path3.basename(folder)}/${titleSlug}${ext}`);
        await sleep(cooldown);
      }
    }
    table.push(row);
  }
  spinner.succeed("Submissions Dumped.");
  const username = (await retry(() => leetcode.graphql({
    query: `query globalData { userStatus { username } }`,
    operationName: "globalData",
    variables: {}
  }), max_retry)).data.userStatus.username;
  const card_url = `https://leetcode.card.workers.dev/?username=${username}`;
  const table_str = table.map(([title, difficulty, tags, solutions]) => {
    return `| ${title} | ${difficulty} | ${tags} | ${[...solutions.entries()].map(([lang, link]) => `[${lang}](${encodeURI(link)})`).join(" / ")} |`;
  }).join("\n");
  fs2.writeFileSync(path3.resolve(dir, "README.md"), README_TEMPLATE.replace("$card_url", card_url).replace("$table", table_str));
}
async function get_list() {
  const ql = `query allQuestionsStatusesRaw { allQuestions: allQuestionsRaw { titleSlug questionId status } }`;
  const json = await retry(() => leetcode.graphql({
    query: ql,
    operationName: "allQuestionsStatusesRaw",
    variables: {}
  }), max_retry);
  const list = json.data.allQuestions;
  const ac = list.filter((item) => item.status === "ac").sort((a, b) => +a.questionId - +b.questionId);
  return { list, ac };
}
async function get_problem(slug) {
  const ql = `
    query questionData($slug: String!) {
        question(titleSlug: $slug) {
            questionFrontendId title content difficulty similarQuestions categoryTitle stats topicTags { name slug }
        }
    }`;
  const json = (await retry(() => leetcode.graphql({
    query: ql,
    operationName: "questionData",
    variables: { slug }
  }), max_retry)).data.question;
  json.similarQuestions = JSON.parse(json.similarQuestions);
  json.stats = JSON.parse(json.stats);
  return json;
}
async function get_submissions(slug) {
  const ql = `
    query Submissions($offset: Int!, $limit: Int!, $last: String, $slug: String!) {
        submissionList(offset: $offset, limit: $limit, lastKey: $last, questionSlug: $slug) {
            hasNext
            submissions { id statusDisplay lang runtime timestamp }
        }
    }`;
  const list = {};
  for (let i = 0; i < 10; i++) {
    const json = await retry(() => leetcode.graphql({
      query: ql,
      operationName: "Submissions",
      variables: { slug, offset: 0 * 20, limit: 20 }
    }), max_retry);
    const { submissions, hasNext } = json.data.submissionList;
    for (const submission of submissions) {
      const { id, statusDisplay, lang, runtime, timestamp } = submission;
      if (statusDisplay !== "Accepted") {
        continue;
      }
      if (!list[lang] || parseInt(runtime) < parseInt(list[lang].runtime)) {
        list[lang] = { id, timestamp, runtime };
      }
    }
    if (submissions.length === 0 || !hasNext) {
      break;
    }
  }
  return list;
}
async function get_submission(id) {
  var _a;
  const res = await retry(() => fetch(`${LEETCODE_DETAIL_BASE}${id}/`, {
    headers: {
      Cookie: `LEETCODE_SESSION=${leetcode.credential.session}`,
      Referer: LEETCODE_BASE
    }
  }), max_retry);
  const raw = await res.text();
  const data = (_a = raw.match(/var pageData = ({[^]+?});/)) == null ? void 0 : _a[1];
  const json = new Function("return " + data)();
  const result = {
    runtime: +json.runtime,
    runtime_distribution: json.runtimeDistributionFormatted ? JSON.parse(json.runtimeDistributionFormatted).distribution.map((item) => [+item[0], item[1]]) : null,
    runtime_percentile: 0,
    memory: +json.memory,
    memory_distribution: json.memoryDistributionFormatted ? JSON.parse(json.memoryDistributionFormatted).distribution.map((item) => [+item[0], item[1]]) : null,
    memory_percentile: 0,
    code: json.submissionCode
  };
  if (result.runtime_distribution) {
    result.runtime_percentile = result.runtime_distribution.reduce((acc, [usage, p]) => acc + (usage >= result.runtime ? p : 0), 0);
  }
  if (result.memory_distribution) {
    result.memory_percentile = result.memory_distribution.reduce((acc, [usage, p]) => acc + (usage >= result.memory / 1e3 ? p : 0), 0);
  }
  return result;
}

// src/index.ts
config();
var package_meta = get_package_meta();
program.version(chalk2`{cyanBright ${package_meta.name}} {yellowBright v${package_meta.version}}`);
process.stdout.isTTY = true;
program.option("-s, --session <session>", "Your LeetCode Session", process.env.LEETCODE_SESSION || "process.env.LEETCODE_SESSION").option("-o, --output <path>", "Output Dir", path4.resolve("leetcode")).option("-c, --clean", "Clean Output Dir Before Start", true).option("-cd, --cooldown <ms>", "Cooldown Between Actions, in ms", "1000").option("-t, --timezone <timezone>", "Your Timezone", "Asia/Taipei").option("-p, --pure", "Pure Mode, No Additional Informations to Add", false).option("-r, --retry <times>", "Times to Retry When Fail", "3").action(async () => {
  const opts = program.opts();
  await dump(opts.session, opts.output, opts.clean, +opts.cooldown, opts.timezone, opts.pure, +opts.retry);
});
program.parse();
